==> xl-repository-tck-9.7.15-sources.jar: com/xebialabs/xlplatform/xlrepository/tck/suites/RepositoryTypeSystemSuite.scala <==
package com.xebialabs.xlplatform.xlrepository.tck.suites

import java.util
import java.util.{Date, List => JList}

import com.xebialabs.deployit.core._
import com.xebialabs.deployit.plugin.api.reflect.Type
import com.xebialabs.deployit.plugin.api.udm.ConfigurationItem
import com.xebialabs.deployit.util.PasswordEncrypter
import com.xebialabs.xlplatform.xlrepository.tck.RepositoryFactory
import com.xebialabs.xlplatform.xlrepository.tck.cis._
import com.xebialabs.xlplatform.xlrepository.tck.util.CiHelper._
import com.xebialabs.xlplatform.xlrepository.tck.util.RepositorySuiteBase
import com.xebialabs.xlplatform.xlrepository.tck.util.RepositorySuiteBase._

import scala.collection.JavaConverters._

trait RepositoryTypeSystemSuite extends RepositorySuiteBase {
  self: RepositoryFactory[_] =>

  private val gen = generator.genFor("tck-typesystem")

  describe("The XL Repository TypeSystem Suite") {
    it("should encrypt password property in the repository") {
      val password: Password = ciHelper.constructCi("repo-tck.Password", gen.nextId())
      password.password = "Secret"
      repository.create(password)
      password property "password" should not be (storedInRepository equal "Secret")
      password property "password" shouldBe (storedInRepository startingWith PasswordEncrypter.getInstance().prefix())
    }

    it("should not persist/read hidden properties") {
      val hidden: Hidden = ciHelper.constructCi("repo-tck.Hidden", gen.nextId())
      hidden.hidden = "ShowMe!"
      repository.create(hidden)
      val read: Hidden = repository.read(hidden.getId)
      read.hidden shouldEqual "Hidden"
    }

    it("should not persist/read transient properties") {
      val transient: Transient = ciHelper.constructCi("repo-tck.Transient", gen.nextId())
      transient.transient = "PersistMe!"
      repository.create(transient)
      val read: Transient = repository.read(transient.getId)
      read.transient shouldEqual "Transient"
    }

    it("should write List<String> as StringListView") {
      val ci: ListOfString = ciHelper.constructCi("repo-tck.ListOfString", gen.nextId())
      ci.prop = List("foo", "bar").asJava
      repository.create(ci)
      val read: ListOfString = repository.read(ci.getId)
      read.prop shouldBe a[ListOfStringView]
      val view = read.prop.asInstanceOf[ListOfStringView].getWrapped
      view.get(0) should not be 'encrypted
      view.get(1) should not be 'encrypted
    }

    it("should maintain order for a List<String>") {
      val ci: ListOfString = ciHelper.constructCi("repo-tck.ListOfString", gen.nextId())
      ci.prop = List("foo", "bar").asJava
      repository.create(ci)
      val read: ListOfString = repository.read(ci.getId)
      read.prop should contain inOrder("foo", "bar")
      read.prop = read.prop.asScala.reverse.asJava
      read.prop.add("baz")
      repository.update(read)
      val read1: ListOfString = repository.read(ci.getId, false)
      read1.prop should contain inOrder("bar", "foo", "baz")
    }

    it("should write Set<String> as StringSetView") {
      val ci: SetOfString = ciHelper.constructCi("repo-tck.SetOfString", gen.nextId())
      ci.prop = Set("foo", "bar").asJava
      repository.create(ci)
      val read: SetOfString = repository.read(ci.getId)
      read.prop shouldBe a[SetOfStringView]
      val view = read.prop.asInstanceOf[SetOfStringView].getWrapped
      view.forEach( x => x should not be 'encrypted )
    }

    it("should write Map<String, String> as MapStringStringView") {
      val ci: MapStringString = ciHelper.constructCi("repo-tck.MapStringString", gen.nextId())
      ci.prop = Map("key" -> "value").asJava
      repository.create(ci)
      val read: MapStringString = repository.read(ci.getId)
      read.prop shouldBe a[MapStringStringView]
      val view = read.prop.asInstanceOf[MapStringStringView].getWrapped
      view.get("key") should not be 'encrypted
    }

    it("should write password List<String> as encrypted StringListView") {
      val ci: PasswordListOfString = ciHelper.constructCi("repo-tck.PasswordListOfString", gen.nextId())
      ci.prop = List("foo", "bar").asJava
      repository.create(ci)
      val read: PasswordListOfString = repository.read(ci.getId)
      read.prop shouldBe a[ListOfStringView]
      val view = read.prop.asInstanceOf[ListOfStringView].getWrapped
      view.get(0) shouldBe 'encrypted
      view.get(1) shouldBe 'encrypted
    }

    it("should write password Set<String> as encrypted StringSetView") {
      val ci: PasswordSetOfString = ciHelper.constructCi("repo-tck.PasswordSetOfString", gen.nextId())
      ci.prop = Set("foo", "bar").asJava
      repository.create(ci)
      val read: PasswordSetOfString = repository.read(ci.getId)
      read.prop shouldBe a[SetOfStringView]
      val view = read.prop.asInstanceOf[SetOfStringView].getWrapped
      view.forEach( x => x shouldBe 'encrypted )
    }

    it("should write password Map<String, String> as encrypted MapStringStringView") {
      val ci: PasswordMapStringString = ciHelper.constructCi("repo-tck.PasswordMapStringString", gen.nextId())
      ci.prop = Map("key" -> "value").asJava
      repository.create(ci)
      val read: PasswordMapStringString = repository.read(ci.getId)
      read.prop shouldBe a[MapStringStringView]
      val view = read.prop.asInstanceOf[MapStringStringView].getWrapped
      view.get("key") shouldBe 'encrypted
    }

    it("should keep partially encrypted SetOfStringView intact") {
      val ci: SetOfString = ciHelper.constructCi("repo-tck.SetOfString", gen.nextId())
      ci.prop = new SetOfStringView()
      ci.prop.asInstanceOf[SetOfStringView].addEncrypted("secret")
      ci.prop.add("plain")
      repository.create(ci)
      val read: SetOfString = repository.read(ci.getId)
      read.prop should have size 2
      read.prop.asInstanceOf[SetOfStringView].getWrapped should contain(new StringValue("plain"))
      read.prop.asInstanceOf[SetOfStringView].getWrapped should contain(new EncryptedStringValue("secret"))
    }

    it("should keep partially encrypted ListOfStringView intact") {
      val ci: ListOfString = ciHelper.constructCi("repo-tck.ListOfString", gen.nextId())
      ci.prop = new ListOfStringView()
      ci.prop.asInstanceOf[ListOfStringView].addEncrypted("secret")
      ci.prop.add("plain")
      repository.create(ci)
      val read: ListOfString = repository.read(ci.getId)
      read.prop should have size 2
      read.prop.asInstanceOf[ListOfStringView].getWrapped should contain(new StringValue("plain"))
      read.prop.asInstanceOf[ListOfStringView].getWrapped should contain(new EncryptedStringValue("secret"))
    }

    it("should keep partially encrypted MapStringStringView intact") {
      val ci: MapStringString = ciHelper.constructCi("repo-tck.MapStringString", gen.nextId())
      ci.prop = new MapStringStringView()
      ci.prop.asInstanceOf[MapStringStringView].put("password", new EncryptedStringValue("secret"))
      ci.prop.put("random", "plain")
      repository.create(ci)
      val read: MapStringString = repository.read(ci.getId)
      read.prop should have size 2
      read.prop.asInstanceOf[MapStringStringView].getWrapped should contain value new StringValue("plain")
      read.prop.asInstanceOf[MapStringStringView].getWrapped should contain value new EncryptedStringValue("secret")
    }

    it("should force encryption of list of string if non-encrypted values stored in repo are read into password property") {
      val ci: PasswordListOfString = ciHelper.constructCi("repo-tck.PasswordListOfString", gen.nextId())
      repository.create(ci)
      setStoredValue(ci, "prop", List("foo", "bar"))

      val read: PasswordListOfString = repository.read(ci.getId)
      read.prop.asInstanceOf[ListOfStringView].getWrapped.forEach { x =>
        x shouldBe 'encrypted
      }
    }

    it("should force encryption of set of string if non-encrypted values stored in repo are read into password property") {
      val ci: PasswordSetOfString = ciHelper.constructCi("repo-tck.PasswordSetOfString", gen.nextId())
      repository.create(ci)
      setStoredValue(ci, "prop", Set("foo", "bar"))

      val read: PasswordSetOfString = repository.read(ci.getId)
      read.prop.asInstanceOf[SetOfStringView].getWrapped.forEach { x =>
        x shouldBe 'encrypted
      }
    }

    it("should force encryption of map of string if non-encrypted values stored in repo are read into password property") {
      val ci: PasswordMapStringString = ciHelper.constructCi("repo-tck.PasswordMapStringString", gen.nextId())
      repository.create(ci)
      setStoredValue(ci, "prop", Map("password" -> "secret", "another_secret" -> "hidden"))

      val read: PasswordMapStringString = repository.read(ci.getId)
      read.prop.asInstanceOf[MapStringStringView].getWrapped.values().forEach { x =>
        x shouldBe 'encrypted
      }
    }

    it("should read and update convert value for property that has changed kind from String to list of string") {
      val ci = ciHelper.constructConfigurationItem(classOf[AllProperties], gen.nextId())
      repository.create(ci)

      setStoredValue(ci, "stringList", "listValue")

      // read it as a different type to test the backwards compatibility of String->ListOfString kind
      var readCi = repository.read[AllProperties](ci.getId)
      readCi.stringList should have size 1
      readCi.stringList.get(0) should equal("listValue")

      readCi.stringList = new util.ArrayList[String](List("listValue", "container-by-container").asJava)
      repository.update(readCi)

      readCi = repository.read(ci.getId)
      readCi.stringList.size shouldBe 2
      readCi.stringList.get(0) should equal("listValue")
      readCi.stringList.get(1) should equal("container-by-container")
    }

    it("should not fail on absent list of ConfigurationItem containment property") {
      val `type` = Type.valueOf(classOf[AsContainmentList])
      val configurationItem = `type`.getDescriptor.newInstance[AsContainmentList](gen.nextId())
      repository.create(configurationItem)

      removeProperty(configurationItem.getId, "asContainmentList")
      hasProperty(configurationItem.getId, "asContainmentList") shouldBe false

      val read = repository.read[AsContainmentList](configurationItem.getId, false)
      read.getProperty("asContainmentList").asInstanceOf[JList[ConfigurationItem]] should have size 0
    }

    it("should persist a property of kind Date") {
      val ci: DateCi = ciHelper.constructConfigurationItem(classOf[DateCi], gen.nextId())
      val date = new Date()
      ci.prop = date
      repository.create(ci)

      val readCi = repository.read[DateCi](ci.getId)
      readCi.prop shouldBe date
    }

    it("should persist a property of kind Enum") {
      val ci: SampleCi = ciHelper.constructConfigurationItem(classOf[SampleCi],
        gen.nextId().replace("Configuration", "Infrastructure"))
      ci.setEnumProperty(SampleCi.SampleEnum.TWO)
      repository.create(ci)

      val readCi = repository.read[SampleCi](ci.getId)
      readCi.enumProperty shouldBe SampleCi.SampleEnum.TWO
    }
  }
}

==> xl-repository-tck-22.0.0-1231.113-sources.jar: com/xebialabs/xlplatform/xlrepository/tck/suites/RepositoryTypeSystemSuite.scala <==
package com.xebialabs.xlplatform.xlrepository.tck.suites

import java.util
import java.util.{Date, List => JList}

import com.xebialabs.deployit.core._
import com.xebialabs.deployit.plugin.api.reflect.Type
import com.xebialabs.deployit.plugin.api.udm.ConfigurationItem
import com.xebialabs.deployit.util.PasswordEncrypter
import com.xebialabs.xlplatform.xlrepository.tck.RepositoryFactory
import com.xebialabs.xlplatform.xlrepository.tck.cis._
import com.xebialabs.xlplatform.xlrepository.tck.util.CiHelper._
import com.xebialabs.xlplatform.xlrepository.tck.util.RepositorySuiteBase
import com.xebialabs.xlplatform.xlrepository.tck.util.RepositorySuiteBase._

import scala.jdk.CollectionConverters._

trait RepositoryTypeSystemSuite extends RepositorySuiteBase {
  self: RepositoryFactory[_] =>

  private val gen = generator.genFor("tck-typesystem")

  describe("The XL Repository TypeSystem Suite") {
    it("should encrypt password property in the repository") {
      val password: Password = ciHelper.constructCi("repo-tck.Password", gen.nextId())
      password.password = "Secret"
      repository.create(password)
      password property "password" should not be (storedInRepository equal "Secret")
      password property "password" shouldBe (storedInRepository startingWith PasswordEncrypter.getInstance().prefix())
    }

    it("should not persist/read hidden properties") {
      val hidden: Hidden = ciHelper.constructCi("repo-tck.Hidden", gen.nextId())
      hidden.hidden = "ShowMe!"
      repository.create(hidden)
      val read: Hidden = repository.read(hidden.getId)
      read.hidden shouldEqual "Hidden"
    }

    it("should not persist/read transient properties") {
      val transient: Transient = ciHelper.constructCi("repo-tck.Transient", gen.nextId())
      transient.transient = "PersistMe!"
      repository.create(transient)
      val read: Transient = repository.read(transient.getId)
      read.transient shouldEqual "Transient"
    }

    it("should write List<String> as StringListView") {
      val ci: ListOfString = ciHelper.constructCi("repo-tck.ListOfString", gen.nextId())
      ci.prop = List("foo", "bar").asJava
      repository.create(ci)
      val read: ListOfString = repository.read(ci.getId)
      read.prop shouldBe a[ListOfStringView]
      val view = read.prop.asInstanceOf[ListOfStringView].getWrapped
      view.get(0) should not be Symbol("encrypted")
      view.get(1) should not be Symbol("encrypted")
    }

    it("should maintain order for a List<String>") {
      val ci: ListOfString = ciHelper.constructCi("repo-tck.ListOfString", gen.nextId())
      ci.prop = List("foo", "bar").asJava
      repository.create(ci)
      val read: ListOfString = repository.read(ci.getId)
      read.prop should contain.inOrder("foo", "bar")
      read.prop = read.prop.asScala.reverse.asJava
      read.prop.add("baz")
      repository.update(read)
      val read1: ListOfString = repository.read(ci.getId, false)
      read1.prop should contain.inOrder("bar", "foo", "baz")
    }

    it("should write Set<String> as StringSetView") {
      val ci: SetOfString = ciHelper.constructCi("repo-tck.SetOfString", gen.nextId())
      ci.prop = Set("foo", "bar").asJava
      repository.create(ci)
      val read: SetOfString = repository.read(ci.getId)
      read.prop shouldBe a[SetOfStringView]
      val view = read.prop.asInstanceOf[SetOfStringView].getWrapped
      view.forEach( x => x should not be Symbol("encrypted") )
    }

    it("should write Map<String, String> as MapStringStringView") {
      val ci: MapStringString = ciHelper.constructCi("repo-tck.MapStringString", gen.nextId())
      ci.prop = Map("key" -> "value").asJava
      repository.create(ci)
      val read: MapStringString = repository.read(ci.getId)
      read.prop shouldBe a[MapStringStringView]
      val view = read.prop.asInstanceOf[MapStringStringView].getWrapped
      view.get("key") should not be Symbol("encrypted")
    }

    it("should write password List<String> as encrypted StringListView") {
      val ci: PasswordListOfString = ciHelper.constructCi("repo-tck.PasswordListOfString", gen.nextId())
      ci.prop = List("foo", "bar").asJava
      repository.create(ci)
      val read: PasswordListOfString = repository.read(ci.getId)
      read.prop shouldBe a[ListOfStringView]
      val view = read.prop.asInstanceOf[ListOfStringView].getWrapped
      view.get(0) shouldBe Symbol("encrypted")
      view.get(1) shouldBe Symbol("encrypted")
    }

    it("should write password Set<String> as encrypted StringSetView") {
      val ci: PasswordSetOfString = ciHelper.constructCi("repo-tck.PasswordSetOfString", gen.nextId())
      ci.prop = Set("foo", "bar").asJava
      repository.create(ci)
      val read: PasswordSetOfString = repository.read(ci.getId)
      read.prop shouldBe a[SetOfStringView]
      val view = read.prop.asInstanceOf[SetOfStringView].getWrapped
      view.forEach( x => x shouldBe Symbol("encrypted") )
    }

    it("should write password Map<String, String> as encrypted MapStringStringView") {
      val ci: PasswordMapStringString = ciHelper.constructCi("repo-tck.PasswordMapStringString", gen.nextId())
      ci.prop = Map("key" -> "value").asJava
      repository.create(ci)
      val read: PasswordMapStringString = repository.read(ci.getId)
      read.prop shouldBe a[MapStringStringView]
      val view = read.prop.asInstanceOf[MapStringStringView].getWrapped
      view.get("key") shouldBe Symbol("encrypted")
    }

    it("should keep partially encrypted SetOfStringView intact") {
      val ci: SetOfString = ciHelper.constructCi("repo-tck.SetOfString", gen.nextId())
      ci.prop = new SetOfStringView()
      ci.prop.asInstanceOf[SetOfStringView].addEncrypted("secret")
      ci.prop.add("plain")
      repository.create(ci)
      val read: SetOfString = repository.read(ci.getId)
      read.prop should have size 2
      read.prop.asInstanceOf[SetOfStringView].getWrapped should contain(new StringValue("plain"))
      read.prop.asInstanceOf[SetOfStringView].getWrapped should contain(new EncryptedStringValue("secret"))
    }

    it("should keep partially encrypted ListOfStringView intact") {
      val ci: ListOfString = ciHelper.constructCi("repo-tck.ListOfString", gen.nextId())
      ci.prop = new ListOfStringView()
      ci.prop.asInstanceOf[ListOfStringView].addEncrypted("secret")
      ci.prop.add("plain")
      repository.create(ci)
      val read: ListOfString = repository.read(ci.getId)
      read.prop should have size 2
      read.prop.asInstanceOf[ListOfStringView].getWrapped should contain(new StringValue("plain"))
      read.prop.asInstanceOf[ListOfStringView].getWrapped should contain(new EncryptedStringValue("secret"))
    }

    it("should keep partially encrypted MapStringStringView intact") {
      val ci: MapStringString = ciHelper.constructCi("repo-tck.MapStringString", gen.nextId())
      ci.prop = new MapStringStringView()
      ci.prop.asInstanceOf[MapStringStringView].put("password", new EncryptedStringValue("secret"))
      ci.prop.put("random", "plain")
      repository.create(ci)
      val read: MapStringString = repository.read(ci.getId)
      read.prop should have size 2
      read.prop.asInstanceOf[MapStringStringView].getWrapped should contain value new StringValue("plain")
      read.prop.asInstanceOf[MapStringStringView].getWrapped should contain value new EncryptedStringValue("secret")
    }

    it("should force encryption of list of string if non-encrypted values stored in repo are read into password property") {
      val ci: PasswordListOfString = ciHelper.constructCi("repo-tck.PasswordListOfString", gen.nextId())
      repository.create(ci)
      setStoredValue(ci, "prop", List("foo", "bar"))

      val read: PasswordListOfString = repository.read(ci.getId)
      read.prop.asInstanceOf[ListOfStringView].getWrapped.forEach { x =>
        x shouldBe Symbol("encrypted")
      }
    }

    it("should force encryption of set of string if non-encrypted values stored in repo are read into password property") {
      val ci: PasswordSetOfString = ciHelper.constructCi("repo-tck.PasswordSetOfString", gen.nextId())
      repository.create(ci)
      setStoredValue(ci, "prop", Set("foo", "bar"))

      val read: PasswordSetOfString = repository.read(ci.getId)
      read.prop.asInstanceOf[SetOfStringView].getWrapped.forEach { x =>
        x shouldBe Symbol("encrypted")
      }
    }

    it("should force encryption of map of string if non-encrypted values stored in repo are read into password property") {
      val ci: PasswordMapStringString = ciHelper.constructCi("repo-tck.PasswordMapStringString", gen.nextId())
      repository.create(ci)
      setStoredValue(ci, "prop", Map("password" -> "secret", "another_secret" -> "hidden"))

      val read: PasswordMapStringString = repository.read(ci.getId)
      read.prop.asInstanceOf[MapStringStringView].getWrapped.values().forEach { x =>
        x shouldBe Symbol("encrypted")
      }
    }

    it("should read and update convert value for property that has changed kind from String to list of string") {
      val ci = ciHelper.constructConfigurationItem(classOf[AllProperties], gen.nextId())
      repository.create(ci)

      setStoredValue(ci, "stringList", "listValue")

      // read it as a different type to test the backwards compatibility of String->ListOfString kind
      var readCi = repository.read[AllProperties](ci.getId)
      readCi.stringList should have size 1
      readCi.stringList.get(0) should equal("listValue")

      readCi.stringList = new util.ArrayList[String](List("listValue", "container-by-container").asJava)
      repository.update(readCi)

      readCi = repository.read(ci.getId)
      readCi.stringList.size shouldBe 2
      readCi.stringList.get(0) should equal("listValue")
      readCi.stringList.get(1) should equal("container-by-container")
    }

    it("should not fail on absent list of ConfigurationItem containment property") {
      val `type` = Type.valueOf(classOf[AsContainmentList])
      val configurationItem = `type`.getDescriptor.newInstance[AsContainmentList](gen.nextId())
      repository.create(configurationItem)

      removeProperty(configurationItem.getId, "asContainmentList")
      hasProperty(configurationItem.getId, "asContainmentList") shouldBe false

      val read = repository.read[AsContainmentList](configurationItem.getId, false)
      read.getProperty("asContainmentList").asInstanceOf[JList[ConfigurationItem]] should have size 0
    }

    it("should persist a property of kind Date") {
      val ci: DateCi = ciHelper.constructConfigurationItem(classOf[DateCi], gen.nextId())
      val date = new Date()
      ci.prop = date
      repository.create(ci)

      val readCi = repository.read[DateCi](ci.getId)
      readCi.prop shouldBe date
    }

    it("should persist a property of kind Enum") {
      val ci: SampleCi = ciHelper.constructConfigurationItem(classOf[SampleCi],
        gen.nextId().replace("Configuration", "Infrastructure"))
      ci.setEnumProperty(SampleCi.SampleEnum.TWO)
      repository.create(ci)

      val readCi = repository.read[SampleCi](ci.getId)
      readCi.enumProperty shouldBe SampleCi.SampleEnum.TWO
    }
  }
}
